/**********************************************************************************/
/*                                                                                */
/* Copyright (c) 2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved     */
/*                                                                                */
/**********************************************************************************/
/*
 *   Process ARM Logs
 *
 */

%macro LogARMAnalyzer();
   %local mname;
   %let mname=LogArmAnalyzer;
   %evcomment((&MNAME) ...initializing  serverType=[&serverType]);

   %let logARMAnalyzer_Debug=0;
   %LogUtilMacroStart(logARMAnalyzer,logARMAnalyzer_Debug);

   %let logdir=;
   %let dirFilter=^arm4_{1}.*\.log*;

   /*
    *  The filter is specified as a regular expression. Changed in 9.2 
    *  for OLAP Server, for OLAP Server, Server type is now OLAPServer9.2
 
   %if %index(%upcase(&serverType),OLAPSERVER) > 0 %then %do;
      %let dirFilter=^arm4_{1}.*\.log*;
   %end;
   %else %do;
      %let dirFilter=^arm4_{1}.*\.log*;
   %end;
   */
   
   %jobheader(jobname=&MNAME, stepname=&serverType, stepdesc=%str(&dirFilter), runid=&runid);

   %*********************************************************;
   %* Uses lz loglist created from loggetlogdirs;
   %*********************************************************;
   data _null_;
      retain buildempty;
      set loglist end=last;
      %if &evdebug gt 0 %then %do;
         putlog "DEBUG: (&MNAME) Stepping thru loglist dirfilter [&dirfilter.] " pathlev4=;
      %end; 
      if (servertype NE 'MetadataServer' AND
             appcontext NE 'nocontext') AND
             logdir EQ 'PerfLogs' then do;
         if servertype =: "&servertype." then do;
            %if &evdebug gt 0 %then %do;
               putlog "DEBUG: (&MNAME) Matched ServerType [&servertype] to logdirectory serverType" serverType=;
            %end; 
            buildempty=1;
            call execute('%nrstr(%logGetLognames('||trim(pathlev4)||',&dirFilter.,minDatetime=&mindatetime.))');
            call execute('%nrstr(%armanalyzer1(logdir='||trim(pathlev4)||',appserver='||trim(appcontext)||',platform='||trim(machine)||'))');
            /* %LogGetLognames(&logdir.,&dirFilter.,minDateTime=&minDateTime.); */
         end;
      end;
      if last and buildempty eq . then do;
         putlog 'last buildempty' buildempty=;
         call symput('numlogs','0');
         call execute('%nrstr(%armanalyzer1(logdir='||trim(pathlev4)||',appserver='||trim(appcontext)||'))');
     end;
   run;
   %LogUtilMacroEnd(logARMAnalyzer,&logARMAnalyzer_Debug.);
   
%mend logarmanalyzer;

%macro armanalyzer1(logdir=,appserver=,platform=);

   %evcomment(Platform [&platform] Context [&appserver], Total ARM logs [&numlogs.] with filter [&dirFilter.]);

   %do i=1 %to &numlogs;
      %evcomment(INFO: (&MNAME) Processing loop [&i.] of [&numlogs.]);
 
	/*
	 *  Some of our log names are long (too long to be a table name) and some contain special characters.
	 *  Apply some rules to try to shorten it and remove the special characters.
	 *
	 *  Use the entire name of the file being read as the sourceId.
	 */
	 
	 %let logtable=&loglib..%sysfunc(translate(&&log&i.,'_','.'));
	 %let sourceId=&&logf&i..;

	/*
	 *  We used to just pass the filename (without extension) and assumed we knew the format.
	 *  However, the more different types of logs we process, the less we know about the format, so
	 *  now the macro variable has the entire filename (including extension) in it.
	 */

	 %let logfile=%trim(&logdir.)/%trim(&&logf&i..);
	 
	/*
	 *  If we are are running this process on a regular basis, we need to look up the last timestamp of the record
	 *  processed for this log.  We will then use this as the minimum datetime stamp to process.
	 *
	 *  Note: We purposely use the minDateTime to query the logs and this "log specific" time stamp to process records.
	 *        We do this because log lines might be cached, so that if we were to just use the minDateTime stamp for both
	 *        getting the list of logs to process and which records to process within that log, we may miss records.
	 */

      %global logMinDateTime;
      %logDTGetLogInfo(&logfile.,&outlib.,&minDateTime.,logMinDateTime);
       
      %if &evdebug GT 0 %then %do;
        %evcomment(INFO: (&MNAME) runID [&runid.], logtable [&logtable.], 
           sourceID [&sourceid.] minDateTime [&minDateTime] logMinDateTime [&logmindatetime.]);
      %end;  

      %*********************************************************;
	 %* run LogARMProcessLog;
      %*********************************************************;
	 %LogARMProcessLog(logfile=&logfile.,
		sourceId=&sourceId.,
		loglib=&loglib.,
		append=0,minDateTime=&logMinDateTime.);

      %*********************************************************;
      %* Get the max time processed and save the updated timestamp processing information;
      %*********************************************************;
   	 %logDTUpdateLogInfo(&logfile.,&loglib..transactions,startdt,&loglib.,minDateTime=&logMinDateTime.,
   	    platform=&platform,context=&appserver,servertype=&servertype);

      %*********************************************************;
      %* Append it to the accumulation data area;
      %*********************************************************;
      %if %sysfunc(exist(&loglib..transactionsAll)) NE 1 %then %do;
         %put INFO: creating [&loglib..transactionsALL] *****;
	    proc datasets lib=&loglib. nolist;
	       change transactions=transactionsall;
	    run;
	    quit;
	 %end;
	 %else %do;
	    proc append data=&loglib..transactions base=&loglib..transactionsAll;
	    run;
      %end;

      %*********************************************************;
      %* enable debug save intermediate datasets;
      %*********************************************************;
      %let armsave=armsave;
        
      %if (%sysfunc(libref(&armSave.))=0) %then %do;
         data %scan(&armsave.,1,'.').transactions&i.;
            set &loglib..transactions;
         run;   
      %end;      
   %end; /* of do loop */

%mend armanalyzer1;
